@@ -13,10 +13,10 @@ from dataclasses import dataclass
 from typing import Mapping, Optional, Any
 
 import yarl
-from aiohttp import ClientSession, BasicAuth
+from aiohttp import ClientSession, BasicAuth, FormData, hdrs
 from async_tools import AsyncOnStop, DummyAsyncContextManager
 from init_helpers.custom_json import custom_dumps, Jsonable
-from multidict import MultiMapping
+from multidict import MultiMapping, CIMultiDict
 
 from http_tools import HttpClient
 
@@ -32,7 +32,7 @@ class HttpServerConnector(AsyncOnStop):
         location: str = ""  # deprecated, use url instead
         url: str = ""  # use it
         timeout_sec: float = 100
-        max_connections: int = None
+        max_connections: int = 10
         proxy_url: str = None
 
         def __post_init__(self):
@@ -54,11 +54,13 @@ class HttpServerConnector(AsyncOnStop):
     @dataclass
     class Context:
         session: ClientSession
+        instance_id: str = ""  # empty string - backward compatibility
 
     def __init__(self, config: Config, context: Context):
         logger.info(f"initiating {type(self).__name__}, {config.url}")
 
         self.config = config
+        self.context = context
         self._requester = HttpClient(context.session)
         self._semaphore =\
             Semaphore(self.config.max_connections) if self.config.max_connections else DummyAsyncContextManager()
@@ -68,11 +70,7 @@ class HttpServerConnector(AsyncOnStop):
         await self._requester._on_stop()
 
     def _get_url(self, path: str) -> str:
-        slash = '/'
-        url = str(yarl.URL(self.config.url) / path.lstrip(slash))
-        if path.endswith(slash):
-            url += slash
-        return url
+        return str(yarl.URL(self.config.url) / path.lstrip('/'))
 
     async def post_json(self, path: str, payload: Jsonable, url_query: MultiMapping = None,
                         headers: Optional[Mapping[str, str]] = None, auth: Optional[BasicAuth] = None,
@@ -82,6 +80,7 @@ class HttpServerConnector(AsyncOnStop):
         payload = json.loads(jsom_payload)
         url = self._get_url(path)
         timeout_sec = timeout_sec if timeout_sec else self.config.timeout_sec
+        headers = self._ensure_headers(headers)
         async with self._semaphore:
             result = await self._requester.post(url, params=url_query, json=payload, headers=headers, auth=auth,
                                                 proxy_url=self.config.proxy_url, timeout=timeout_sec)
@@ -94,16 +93,18 @@ class HttpServerConnector(AsyncOnStop):
         payload = json.loads(jsom_payload)
         url = self._get_url(path)
         timeout_sec = timeout_sec if timeout_sec else self.config.timeout_sec
+        headers = self._ensure_headers(headers)
         async with self._semaphore:
             result = await self._requester.put(url, params=url_query, json=payload, headers=headers, auth=auth,
                                                proxy_url=self.config.proxy_url, timeout=timeout_sec)
         return result
 
-    async def post(self, path: str, payload: bytes, url_query: Mapping = None,
+    async def post(self, path: str, payload: bytes | FormData, url_query: Mapping = None,
                    headers: Optional[Mapping[str, str]] = None, auth: Optional[BasicAuth] = None,
                    timeout_sec: Optional[float] = None) -> Any:
         url = self._get_url(path)
         timeout_sec = timeout_sec if timeout_sec else self.config.timeout_sec
+        headers = self._ensure_headers(headers)
         async with self._semaphore:
             result = await self._requester.post(url, params=url_query, data=payload, headers=headers, auth=auth,
                                                 proxy_url=self.config.proxy_url, timeout=timeout_sec)
@@ -114,6 +115,7 @@ class HttpServerConnector(AsyncOnStop):
                      timeout_sec: Optional[float] = None) -> Any:
         url = self._get_url(path)
         timeout_sec = timeout_sec if timeout_sec else self.config.timeout_sec
+        headers = self._ensure_headers(headers)
         async with self._semaphore:
             result = await self._requester.delete(url, params=url_query, data=payload, headers=headers, auth=auth,
                                                   proxy_url=self.config.proxy_url, timeout=timeout_sec)
@@ -124,6 +126,7 @@ class HttpServerConnector(AsyncOnStop):
                   timeout_sec: Optional[float] = None) -> Any:
         url = self._get_url(path)
         timeout_sec = timeout_sec if timeout_sec else self.config.timeout_sec
+        headers = self._ensure_headers(headers)
         async with self._semaphore:
             result = await self._requester.get(url, params=url_query, headers=headers, auth=auth,
                                                proxy_url=self.config.proxy_url, timeout=timeout_sec)
@@ -136,7 +139,15 @@ class HttpServerConnector(AsyncOnStop):
         payload = json.loads(jsom_payload)
         url = self._get_url(path)
         timeout_sec = timeout_sec if timeout_sec else self.config.timeout_sec
+        headers = self._ensure_headers(headers)
         async with self._semaphore:
             result = await self._requester.patch(url, params=url_query, json=payload, headers=headers, auth=auth,
                                                  proxy_url=self.config.proxy_url, timeout=timeout_sec)
         return result
+
+    def _ensure_headers(self, required_headers: Mapping[str, str] | None) -> CIMultiDict[str, str]:
+        headers = CIMultiDict()
+        headers.update(required_headers or {})
+        if self.context.instance_id:
+            headers[hdrs.USER_AGENT] = self.context.instance_id
+        return headers
