@@ -9,6 +9,8 @@ class HttpStatusCode(IntEnum):
     NoContent = 204
     MovedPermanently = 301
     Found = 302
+    SeeOther = 303
+    NotModified = 304
     BadRequest = 400
     Unauthorized = 401
     Forbidden = 403
